use gtk4 as gtk;

use std::{num::ParseFloatError, str::FromStr};

use gtk::{
    prelude::{TextBufferExt, TextViewExt},
    TextBuffer,
};

use crate::view::components::{info_bar::InfoBar, input::Input};

pub fn get_error_message(error_instance: ParseFloatError) -> Option<&'static str> {
    match error_instance.to_string().as_str() {
        "cannot parse float from empty string" => Some("Вы не ввели данные в поле/-я"),
        "invalid float literal" => Some("Вы ввели не корректные данные в поле/-я"),
        _ => None,
    }
}

pub fn parse_fields(all_inputs: Vec<Input>) -> Option<[f64; 6]> {
    let mut values: [f64; 6] = [0f64; 6];

    let info_bar = InfoBar::get_instance();

    for (i, input) in all_inputs.iter().enumerate() {
        let input_text_buffer: TextBuffer = input.clone().get_input().buffer();
        let try_extract_value = f64::from_str(
            input_text_buffer
                .text(
                    &input_text_buffer.start_iter(),
                    &input_text_buffer.end_iter(),
                    false,
                )
                .as_str()
                .trim(),
        );
        match try_extract_value {
            Ok(value) => values[i] = value,
            Err(error) => {
                let error_kind: Option<&str> = get_error_message(error);

                info_bar.set_text_label(error_kind);
                info_bar.show_infobar(5u64);

                return None;
            }
        }
    }

    Some(values)
}
